#include <Servo.h> // Include Servo Library

Servo myservo;  // create servo object to control a servo

// Define pin names in variables
int dataPin = 12;
int enablePin = 11;
int latchPin = 10;
int clockPin = 9;

boolean registers[numOfRegisterPins]; // To store the states of all pins

#define number_of_74hc595s 5 // How many shift registers we are using
#define numOfRegisterPins number_of_74hc595s * 8 // number of all pins on all shift registers
#define numRGBLeds 12 // number of RGB LEDs being used



int redPin[]   = {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33 }; // pin numbers that correspond to the red LEDs
int greenPin[] = {1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31, 34 }; // pin numbers that correspond to the green LEDs
int bluePin[]  = {2, 5, 8, 11, 14, 17, 20, 23, 26, 29, 32, 35 }; // pin numbers that correspond to the blue LEDs
int minutesLED = 0; // to store the number of LED that represents minutes
int hoursLED = 0; // to store the number of LED that represents hours



void setup() {
  myservo.attach(6);  // attaches the servo on pin 6 to the servo object

  // set the 4 logic connections with the shift registers as outputs
  pinMode(dataPin,  OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);

  digitalWrite(enablePin, LOW); // set all outputs to HIGH
  for (int i = numOfRegisterPins - 1; i >=  0; i--) {
    registers[i] = LOW; // set all output states to LOW
  }
  ledOff(0); // Turn off all LEDs with 0 clear delay.

}

void loop() {
  int second = 5000; // 5 seconds
  if(hoursLED > 11){ // every 12 hours, start over
    hoursLED = 0;
  }
  for (byte count2 = 0; count2 < numRGBLeds; count2++) { // repeat what is inside 12 times for one hour

    if(minutesLED > 11){ // every hour, start the minutes LED over
      minutesLED = 0;
    }

    for (byte count1 = 0; count1 < 5; count1++) { // repeat what is inside 5 times for 5 minutes
      for (byte count = 0; count < numRGBLeds; count++) { // update the states of the LEDs every 5 seconds
        
        ledOff(0); // make sure all other LEDs are turned off
        registersWrite(greenPin[hoursLED], HIGH); // turn on hours LED
        registersWrite(bluePin[minutesLED], HIGH); // turn on minutes LED                                                                                                 
        registersWrite(redPin[count], HIGH); // turn on seoconds LED
        delay(seconds); // wait 5 seconds
      }
    }
    minutesLED++; // update minutes LED every 5 minutes
  }

  hoursLED++; // update hours LED every hour

  myservo.write(90); // rotate servo 90 degrees
  delay(1000); // wait a second
  myservo.write(0); // go back to the original position

}

void registersWrite(int index, int value) { // this function write a certain value to a certian output
  digitalWrite(enablePin, LOW); // make sure all outputs are high at the beginning
  digitalWrite(latchPin, LOW); // prevents data from being sent to outputs
  registers[index] = value; // set the specified pin to the specified state
  for (int i = numOfRegisterPins - 1; i >=  0; i--) { // go over all register pins
    digitalWrite(clockPin, LOW); // do not update data stored in the register
    int val = registers[i]; // take the state of the ith pin
    digitalWrite(dataPin, val); // send its state to the shift register
    digitalWrite(clockPin, HIGH); // store its state
  }
  digitalWrite(latchPin, HIGH); // output all states
  
}


void ledOff(int clearSpeed) { // this function sets all LEDs off at a chosed clear speed
  for (int i = 0; i <= 35; i++) { // goes over all 36 LEDs and sets them LOW
    registersWrite(i, LOW);
    delay(clearSpeed);
  }
}











